@@ -27,11 +27,13 @@ from apt import Cache
 from apt import debfile
 from gettext import gettext as _
 from mimetypes import guess_type
-from softwarecenter.apt.apthistory import get_apt_history
 from softwarecenter.distro import get_distro
 from softwarecenter.enums import *
+from softwarecenter.paths import *
 from softwarecenter.utils import *
 
+LOG = logging.getLogger(__name__)
+
 # this is a very lean class as its used in the main listview
 # and there are a lot of application objects in memory
 class Application(object):
@@ -46,7 +48,10 @@ class Application(object):
             raise ValueError("Need either appname or pkgname or request")
         # defaults
         self.pkgname = pkgname.replace("$kernel", os.uname()[2])
-        self.appname = appname
+        if appname:
+            self.appname = unicode(appname)
+        else:
+            self.appname = ''
         # the request can take additional "request" data like apturl
         # strings or the path of a local deb package
         self.request = request
@@ -70,6 +75,18 @@ class Application(object):
         """ return a new AppDetails object for this application """
         return AppDetails(db, application=self)
 
+    def get_untranslated_app(self, db):
+        """ return a Application object with the untranslated application
+            name 
+        """
+        try:
+            doc = db.get_xapian_document(self.appname, self.pkgname)
+        except IndexError:
+            return self
+        untranslated_application = doc.get_value(XAPIAN_VALUE_APPNAME_UNTRANSLATED)
+        uapp = Application(untranslated_application, self.pkgname)
+        return uapp
+
     @staticmethod
     def get_display_name(db, doc):
         """ Return the application name as it should be displayed in the UI
@@ -101,6 +118,8 @@ class Application(object):
         return self.apps_cmp(self, other)
     def __str__(self):
         return "%s,%s" % (self.appname, self.pkgname)
+    def __repr__(self):
+        return "[Application: appname=%s pkgname=%s]" % (self.appname, self.pkgname)
     @staticmethod
     def apps_cmp(x, y):
         """ sort method for the applications """
@@ -115,11 +134,12 @@ class Application(object):
         else:
             return cmp(x.pkgname, y.pkgname)
 
+
 class DebFileApplication(Application):
     def __init__(self, debfile):
         # deb overrides this
-        if not debfile.endswith(".deb") and not debfile.count('/') >= 2:
-            raise ValueError("Need a deb file, got '%s'" % debfile)
+#        if not debfile.endswith(".deb") and not debfile.count('/') >= 2:
+ #           raise ValueError("Need a deb file, got '%s'" % debfile)
         debname = os.path.splitext(os.path.basename(debfile))[0]
         self.appname = ""
         self.pkgname = debname.split('_')[0].lower()
@@ -141,9 +161,10 @@ class AppDetails(object):
         if not doc and not application:
             raise ValueError, "Need either document or application"
         self._db = db
+        self._db.connect("reopen", self._on_db_reopen)
         self._cache = self._db._aptcache
         self._distro = get_distro()
-        self._history = get_apt_history()
+        self._history = None
         # import here (intead of global) to avoid dbus dependency
         # in update-software-center (that imports application, but
         # never uses AppDetails) LP: #620011
@@ -192,10 +213,18 @@ class AppDetails(object):
                     self._error = _("Not Found")
                     self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
 
-    @property
-    def architecture(self):
+    def same_app(self, other):
+        return self.pkgname == other.pkgname
+
+    def _on_db_reopen(self, db):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_ARCHIVE_ARCH)
+            try:
+                LOG.debug("db-reopen, refreshing docid for %s" % self._app)
+                self._doc = self._db.get_xapian_document(
+                    self._app.appname, self._app.pkgname)
+            except IndexError:
+                LOG.warn("document no longer valid after db reopen")
+                self._doc = None
 
     @property
     def channelname(self):
@@ -241,11 +270,11 @@ class AppDetails(object):
                 if (origin.origin == "Ubuntu" and origin.trusted and origin.component):
                     return origin.component
         # then xapian
-        if self._doc:
+        elif self._doc:
             comp = self._doc.get_value(XAPIAN_VALUE_ARCHIVE_SECTION)
             return comp
         # then apturl requests
-        if not self._doc:
+        else:
             section_matches = re.findall(r'section=([a-z]+)', self._app.request)
             if section_matches:
                 valid_section_matches = []
@@ -287,6 +316,8 @@ class AppDetails(object):
 
     @property
     def icon(self):
+        if self.pkg_state == PKG_STATE_NOT_FOUND:
+            return MISSING_PKG_ICON
         if self._doc:
             return os.path.splitext(self._db.get_iconname(self._doc))[0]
         if not self.summary:
@@ -296,7 +327,7 @@ class AppDetails(object):
     def icon_file_name(self):
         if self._doc:
             return self._db.get_iconname(self._doc)
-            
+    
     @property
     def icon_needs_download(self):
         if self._doc:
@@ -306,8 +337,15 @@ class AppDetails(object):
     def icon_url(self):
         return self._distro.get_downloadable_icon_url(self._cache, self.pkgname, self.icon_file_name)
 
+    @property
+    def cached_icon_file_path(self):
+        if self._doc:
+            return os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR, self._db.get_iconname(self._doc))
+
     @property
     def installation_date(self):
+        from softwarecenter.apt.apthistory import get_apt_history
+        self._history = get_apt_history()
         return self._history.get_installed_date(self.pkgname)
         
     @property
@@ -371,9 +409,11 @@ class AppDetails(object):
             return PKG_STATE_INSTALLING_PURCHASED
 
         # via the pending transactions dict
-        if self._pkg and self.pkgname in self._backend.pending_transactions:
+        if self.pkgname in self._backend.pending_transactions:
             # FIXME: we don't handle upgrades yet
-            if self._pkg.installed:
+            # if there is no self._pkg yet, that means this is a INSTALL
+            # from a previously not-enabled source (like a purchase)
+            if self._pkg and self._pkg.installed:
                 return PKG_STATE_REMOVING
             else:
                 return PKG_STATE_INSTALLING
@@ -396,7 +436,6 @@ class AppDetails(object):
         #  - the repository information is missing (/var/lib/apt/lists empty)
         #  - its a failure in our meta-data (e.g. typo in the pkgname in
         #    the metadata)
-        #  - not available for our architecture
         if not self._pkg:
             if self.channelname:
                 if self._unavailable_channel():
@@ -406,7 +445,7 @@ class AppDetails(object):
                     self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
                     return PKG_STATE_NOT_FOUND
             else:
-                if self.price and self._available_for_our_arch():
+                if self.price:
                     return PKG_STATE_NEEDS_PURCHASE
                 if (self.purchase_date and
                     self._doc.get_value(XAPIAN_VALUE_ARCHIVE_DEB_LINE)):
@@ -414,15 +453,11 @@ class AppDetails(object):
                 if self.component:
                     components = self.component.split('&')
                     for component in components:
-                        if (component and (self._unavailable_component(component_to_check=component) or self._available_for_our_arch())):
+                        if component and self._unavailable_component(component_to_check=component):
                             return PKG_STATE_NEEDS_SOURCE
-                        if component and not self._available_for_our_arch():
-                            self._error_not_found = _("Not available for this type of computer (%s).") % get_current_arch()
-                            return PKG_STATE_NOT_FOUND
-                else:
-                    self._error =  _("Not Found")
-                    self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
-                    return PKG_STATE_NOT_FOUND
+                self._error =  _("Not Found")
+                self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
+                return PKG_STATE_NOT_FOUND
         return PKG_STATE_UNKNOWN
 
     @property
@@ -475,7 +510,10 @@ class AppDetails(object):
     @property
     def version(self):
         if self._pkg:
-            return self._pkg.candidate.version
+            if self._pkg.installed:
+                return self._pkg.installed.version
+            else:
+                return self._pkg.candidate.version
 
     @property
     def warning(self):
@@ -536,24 +574,6 @@ class AppDetails(object):
         available = self._cache.component_available(distro_codename, component)
         return (not available)
 
-    def _available_for_our_arch(self):
-        """ check if the given package is available for our arch """
-        arches = self.architecture
-        # if we don't have a arch entry in the document its available
-        # on all architectures we know about
-        if not arches:
-            return True
-        # check the arch field and support both "," and ";"
-        sep = ","
-        if ";" in arches:
-            sep = ";"
-        elif "," in arches:
-            sep = ","
-        for arch in map(string.strip, arches.split(sep)):
-            if arch == get_current_arch():
-                return True
-        return False
-
     def __str__(self):
         details = []
         details.append("* AppDetails")
@@ -561,7 +581,6 @@ class AppDetails(object):
         details.append("        display_name: %s" % self.display_name)
         details.append("                 pkg: %s" % self.pkg)
         details.append("             pkgname: %s" % self.pkgname)
-        details.append("        architecture: %s" % self.architecture)
         details.append("         channelname: %s" % self.channelname)
         details.append("                 ppa: %s" % self.ppaname)
         details.append("         channelfile: %s" % self.channelfile)
@@ -636,11 +655,6 @@ class AppDetailsDebFile(AppDetails):
         if not self._deb.check():
             self._error = self._deb._failure_string
 
-    @property
-    def architecture(self):
-        if self._deb:
-            return self._deb._sections["Architecture"]
-
     @property
     def description(self):
         if self._deb:
