@@ -25,6 +25,7 @@ import logging
 import os
 import simplejson
 import string
+import shutil
 import sys
 import time
 import urllib
@@ -34,17 +35,15 @@ from ConfigParser import RawConfigParser, NoOptionError
 from gettext import gettext as _
 from glob import glob
 
-
+from softwarecenter.paths import *
 from softwarecenter.enums import *
 from softwarecenter.paths import SOFTWARE_CENTER_ICON_CACHE_DIR
-from softwarecenter.utils import GnomeProxyURLopener
 from softwarecenter.db.database import parse_axi_values_file
 
 from locale import getdefaultlocale
 import gettext
 import cPickle
 
-
 # weights for the different fields
 WEIGHT_DESKTOP_NAME = 10
 WEIGHT_DESKTOP_KEYWORD = 5
@@ -67,26 +66,44 @@ axi_values = parse_axi_values_file()
 cataloged_times = {}
 CF = "/var/lib/apt-xapian-index/cataloged_times.p"
 if os.path.exists(CF):
-    cataloged_times = cPickle.load(open(CF))
+    try:
+        cataloged_times = cPickle.load(open(CF))
+    except EOFError as e:
+        logging.warn("failed to read %s (%s" % (CF, e))
 del CF
 
 class AppInfoParserBase(object):
     """ base class for reading AppInfo meta-data """
 
-    def get_desktop(self, key):
+    MAPPING = {}
+
+    def get_desktop(self, key, translated=True):
         """ get a AppInfo entry for the given key """
     def has_option_desktop(self, key):
         """ return True if there is a given AppInfo info """
-    def get_desktop_categories(self):
-        categories = []
+    def _get_desktop_list(self, key, split_str=";"):
+        result = []
         try:
-            categories_str = self.get_desktop("Categories")
-            for item in categories_str.split(";"):
+            list_str = self.get_desktop(key)
+            for item in list_str.split(split_str):
                 if item:
-                    categories.append(item)
+                    result.append(item)
         except (NoOptionError, KeyError):
             pass
-        return categories
+        return result
+    def _apply_mapping(self, key):
+        # strip away bogus prefixes
+        if key.startswith("X-AppInstall-"):
+            key = key[len("X-AppInstall-"):]
+        if key in self.MAPPING:
+            return self.MAPPING[key]
+        return key
+    def get_desktop_categories(self):
+        return self._get_desktop_list("Categories")
+    def get_desktop_mimetypes(self):
+        if not self.has_option_desktop("MimeType"):
+            return []
+        return self._get_desktop_list("MimeType")
     @property
     def desktopf(self):
         """ return the file that the AppInfo comes from """
@@ -118,23 +135,15 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
         self.origin = "software-center-agent"
         self._apply_exceptions()
     def _apply_exceptions(self):
-        # for items from the agent, we use the full-size screenshot for
-        # the thumbnail and scale it for display, this is done because
-        # we no longer keep thumbnail versions of screenshots on the server
+        # map screenshot to thumbnail
         if (hasattr(self.sca_entry, "screenshot_url") and 
             not hasattr(self.sca_entry, "thumbnail_url")):
-            self.sca_entry.thumbnail_url = self.sca_entry.screenshot_url
+            url = self.sca_entry.screenshot_url.replace(".png", ".thumb.png")
+            self.sca_entry.thumbnail_url = url
         if hasattr(self.sca_entry, "description"):
             self.sca_entry.Comment = self.sca_entry.description.split("\n")[0]
             self.sca_entry.Description = "\n".join(self.sca_entry.description.split("\n")[1:])
-    def _apply_mapping(self, key):
-        # strip away bogus prefixes
-        if key.startswith("X-AppInstall-"):
-            key = key[len("X-AppInstall-"):]
-        if key in self.MAPPING:
-            return self.MAPPING[key]
-        return key
-    def get_desktop(self, key):
+    def get_desktop(self, key, translated=True):
         if key in self.STATIC_DATA:
             return self.STATIC_DATA[key]
         return getattr(self.sca_entry, self._apply_mapping(key))
@@ -158,14 +167,7 @@ class JsonTagSectionParser(AppInfoParserBase):
     def __init__(self, tag_section, url):
         self.tag_section = tag_section
         self.url = url
-    def _apply_mapping(self, key):
-        # strip away bogus prefixes
-        if key.startswith("X-AppInstall-"):
-            key = key[len("X-AppInstall-"):]
-        if key in self.MAPPING:
-            return self.MAPPING[key]
-        return key
-    def get_desktop(self, key):
+    def get_desktop(self, key, translated=True):
         return self.tag_section[self._apply_mapping(key)]
     def has_option_desktop(self, key):
         return self._apply_mapping(key) in self.tag_section
@@ -173,14 +175,65 @@ class JsonTagSectionParser(AppInfoParserBase):
     def desktopf(self):
         return self.url
 
+class AppStreamXMLParser(AppInfoParserBase):
+
+    MAPPING = { 'Name'       : 'name',
+                'Comment'    : 'summary',
+                'Package'    : 'pkgname',
+                'Categories' : 'appcategories',
+                'Keywords'   : 'keywords',
+                'MimeType'   : 'mimetypes',
+                'Icon'       : 'icon',
+              } 
+
+    LISTS = { "appcategories" : "appcategory",
+              "keywords"  : "keyword",
+              "mimetypes" : "mimetype",
+            }
+    
+    def __init__(self, appinfo_xml, xmlfile):
+        self.appinfo_xml = appinfo_xml
+        self.xmlfile = xmlfile
+    def get_desktop(self, key, translated=True):
+        from lxml import etree
+        key = self._apply_mapping(key)
+        if key in self.LISTS:
+            return self._parse_with_lists(key)
+        else:
+            return self._parse_value(key)
+    def _parse_value(self, key):
+        for child in self.appinfo_xml.iter(key):
+            # FIXME: deal with the i18n
+            if child.get("lang"):
+                continue
+            return child.text
+        return None
+    def _parse_with_lists(self, key):
+        l=[]
+        for listroot in self.appinfo_xml.iter(key):
+            for child in listroot.iter(self.LISTS[key]):
+                l.append(child.text)
+        return ",".join(l)
+    def has_option_desktop(self, key):
+        from lxml import etree
+        key = self._apply_mapping(key)
+        return not self.appinfo_xml.find(key) is None
+    @property
+    def desktopf(self):
+        subelm = self.appinfo_xml.find("id")
+        return subelm.text
+
 class DesktopTagSectionParser(AppInfoParserBase):
     def __init__(self, tag_section, tagfile):
         self.tag_section = tag_section
         self.tagfile = tagfile
-    def get_desktop(self, key):
+    def get_desktop(self, key, translated=True):
         # strip away bogus prefixes
         if key.startswith("X-AppInstall-"):
             key = key[len("X-AppInstall-"):]
+        # shortcut
+        if not translated:
+            return self.tag_section[key]
         # FIXME: make i18n work similar to get_desktop
         # first try dgettext
         if "Gettext-Domain" in self.tag_section:
@@ -217,8 +270,14 @@ class DesktopTagSectionParser(AppInfoParserBase):
 class DesktopConfigParser(RawConfigParser, AppInfoParserBase):
     " thin wrapper that is tailored for xdg Desktop files "
     DE = "Desktop Entry"
-    def get_desktop(self, key):
+    def get_desktop(self, key, translated=True):
         " get generic option under 'Desktop Entry'"
+        # never translate the pkgname
+        if key == "X-AppInstall-Package":
+            return self.get(self.DE, key)
+        # shortcut
+        if not translated:
+            return self.get(self.DE, key)
         # first try dgettext
         if self.has_option_desktop("X-Ubuntu-Gettext-Domain"):
             value = self.get(self.DE, key)
@@ -297,6 +356,24 @@ def update_from_var_lib_apt_lists(db, cache, listsdir=None):
             index_app_info_from_parser(parser, db, cache)
     return True
 
+def update_from_appstream_xml(db, cache, xmldir):
+    from lxml import etree
+    context = glib.main_context_default()
+    for appstream_xml in glob(os.path.join(xmldir, "*.xml")):
+        LOG.debug("processing %s" % appstream_xml)
+        # process events
+        while context.pending():
+            context.iteration()
+        tree = etree.parse(open(appstream_xml))
+        root = tree.getroot()
+        if not root.tag == "applications":
+            logging.error("failed to read '%s' excepected Applications root tag" % appstream_xml)
+            continue
+        for appinfo in root.iter("application"):
+            parser = AppStreamXMLParser(appinfo, appstream_xml)
+            index_app_info_from_parser(parser, db, cache)
+    return True
+        
 def update_from_app_install_data(db, cache, datadir=APP_INSTALL_PATH):
     """ index the desktop files in $datadir/desktop/*.desktop """
     context = glib.main_context_default()
@@ -311,7 +388,14 @@ def update_from_app_install_data(db, cache, datadir=APP_INSTALL_PATH):
             index_app_info_from_parser(parser, db, cache)
         except Exception, e:
             # Print a warning, no error (Debian Bug #568941)
-            LOG.warning("error processing: %s %s" % (desktopf, e))
+            LOG.debug("error processing: %s %s" % (desktopf, e))
+            warning_text = _(
+                "The file: '%s' could not be read correctly. The application "
+                "associated with this file will not be included in the "
+                "software catalog. Please consider raising a bug report "
+                "for this issue with the maintainer of that "
+                "application") % desktopf
+            LOG.warning(warning_text)
     return True
 
 def add_from_purchased_but_needs_reinstall_data(purchased_but_may_need_reinstall_list, db, cache):
@@ -351,30 +435,31 @@ def add_from_purchased_but_needs_reinstall_data(purchased_but_may_need_reinstall
             parser = SoftwareCenterAgentParser(item)
             index_app_info_from_parser(parser, db_purchased, cache)
         except Exception, e:
-            logging.exception("error processing: %s " % e)
+            LOG.exception("error processing: %s " % e)
     # add new in memory db to the main db
     db.add_database(db_purchased)
     # return a query
     query = xapian.Query("AH"+PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME)
     return query
 
-def update_from_software_center_agent(db, cache):
+def update_from_software_center_agent(db, cache, ignore_etag=False,
+                                      include_approved_but_unpublished=False):
     """ update index based on the software-center-agent data """
     def _available_cb(sca, available):
         # print "available: ", available
-        logging.debug("available: '%s'" % available)
+        LOG.debug("available: '%s'" % available)
         sca.available = available
     def _error_cb(sca, error):
-        logging.warn("error: %s" % error)
+        LOG.warn("error: %s" % error)
         sca.available = []
     # use the anonymous interface to s-c-agent, scales much better and is
     # much cache friendlier
     from softwarecenter.backend.restfulclient import SoftwareCenterAgentAnonymous
-    sca = SoftwareCenterAgentAnonymous()
+    sca = SoftwareCenterAgentAnonymous(ignore_etag)
     sca.connect("available", _available_cb)
     sca.connect("error", _error_cb)
     sca.available = None
-    sca.query_available()
+    sca.query_available(include_approved_but_unpublished)
     context = glib.main_context_default()
     while sca.available is None:
         while context.pending():
@@ -388,34 +473,59 @@ def update_from_software_center_agent(db, cache):
             # magic channel
             entry.channel = AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME
             # icon is transmited inline
-            # the iconcache gets mightly confused if there is a "." in the name
-            iconname = "sc-agent-%s" % entry.package_name.replace(".", "__")
-            icondata = base64.b64decode(entry.icon_data)
-            open(os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR,
-                              "%s.png" % iconname),"w").write(icondata)
-            entry.icon = iconname
+            if hasattr(entry, "icon_data") and entry.icon_data:
+                icondata = base64.b64decode(entry.icon_data)
+            elif hasattr(entry, "icon_64_data") and entry.icon_64_data:
+                # workaround for scagent bug #740112
+                icondata = base64.b64decode(entry.icon_64_data)
+            else:
+                icondata = ""
+            # write it if we have data
+            if icondata:
+		# the iconcache gets mightly confused if there is a "." in the name
+                iconname = "sc-agent-%s" % entry.package_name.replace(".", "__")
+                open(os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR,
+                                  "%s.png" % iconname),"w").write(icondata)
+                entry.icon = iconname
             # now the normal parser
             parser = SoftwareCenterAgentParser(entry)
             index_app_info_from_parser(parser, db, cache)
         except Exception, e:
-            logging.warning("error processing: %s " % e)
+            LOG.warning("error processing: %s " % e)
     # return true if we have data entries
     return len(sca.available) > 0
         
 def index_app_info_from_parser(parser, db, cache):
         term_generator = xapian.TermGenerator()
+        term_generator.set_database(db)
+        try:
+            # this tests if we have spelling suggestions (there must be
+            # a better way?!?) - this is needed as inmemory does not have
+            # spelling corrections, but it allows setting the flag and will
+            # raise a exception much later
+            db.add_spelling("test")
+            db.remove_spelling("test")
+            # this enables the flag for it (we only reach this line if
+            # the db supports spelling suggestions)
+            term_generator.set_flags(xapian.TermGenerator.FLAG_SPELLING)
+        except xapian.UnimplementedError:
+            pass
         doc = xapian.Document()
         term_generator.set_document(doc)
         # app name is the data
         if parser.has_option_desktop("X-GNOME-FullName"):
             name = parser.get_desktop("X-GNOME-FullName")
+            untranslated_name = parser.get_desktop("X-GNOME-FullName", translated=False)
         else:
             name = parser.get_desktop("Name")
+            untranslated_name = parser.get_desktop("Name", translated=False)
         if name in seen:
             LOG.debug("duplicated name '%s' (%s)" % (name, parser.desktopf))
         seen.add(name)
         doc.set_data(name)
         index_name(doc, name, term_generator)
+        doc.add_value(XAPIAN_VALUE_APPNAME_UNTRANSLATED, untranslated_name)
+
         # check if we should ignore this file
         if parser.has_option_desktop("X-AppInstall-Ignore"):
             ignore = parser.get_desktop("X-AppInstall-Ignore")
@@ -425,8 +535,14 @@ def index_app_info_from_parser(parser, db, cache):
         # package name
         pkgname = parser.get_desktop("X-AppInstall-Package")
         doc.add_term("AP"+pkgname)
+        if '-' in pkgname:
+            # we need this to work around xapian oddness
+            doc.add_term(pkgname.replace('-','_'))
         doc.add_value(XAPIAN_VALUE_PKGNAME, pkgname)
         doc.add_value(XAPIAN_VALUE_DESKTOP_FILE, parser.desktopf)
+        # display name
+        if "display_name" in axi_values:
+            doc.add_value(axi_values["display_name"], name)
         # cataloged_times
         if "catalogedtime" in axi_values:
             if pkgname in cataloged_times:
@@ -457,7 +573,8 @@ def index_app_info_from_parser(parser, db, cache):
         # purchased date
         if parser.has_option_desktop("X-AppInstall-Purchased-Date"):
             date = parser.get_desktop("X-AppInstall-Purchased-Date")
-            doc.add_value(XAPIAN_VALUE_PURCHASED_DATE, str(date))
+            # strip the subseconds from the end of the date string
+            doc.add_value(XAPIAN_VALUE_PURCHASED_DATE, str(date).split(".")[0])
         # deb-line (third party)
         if parser.has_option_desktop("X-AppInstall-Deb-Line"):
             debline = parser.get_desktop("X-AppInstall-Deb-Line")
@@ -466,6 +583,9 @@ def index_app_info_from_parser(parser, db, cache):
         if parser.has_option_desktop("X-AppInstall-PPA"):
             archive_ppa = parser.get_desktop("X-AppInstall-PPA")
             doc.add_value(XAPIAN_VALUE_ARCHIVE_PPA, archive_ppa)
+            # add archive origin data here so that its available even if
+            # the PPA is not (yet) enabled
+            doc.add_term("XOO"+"lp-ppa-%s" % archive_ppa.replace("/", "-"))
         # screenshot (for third party)
         if parser.has_option_desktop("X-AppInstall-Screenshot-Url"):
             url = parser.get_desktop("X-AppInstall-Screenshot-Url")
@@ -487,6 +607,8 @@ def index_app_info_from_parser(parser, db, cache):
         # write out categories
         for cat in parser.get_desktop_categories():
             doc.add_term("AC"+cat.lower())
+        for mime in parser.get_desktop_mimetypes():
+            doc.add_term("AM"+mime.lower())
         # get type (to distinguish between apps and packages
         if parser.has_option_desktop("Type"):
             type = parser.get_desktop("Type")
@@ -499,6 +621,8 @@ def index_app_info_from_parser(parser, db, cache):
         if parser.has_option_desktop("X-AppInstall-Architectures"):
             arches = parser.get_desktop("X-AppInstall-Architectures")
             doc.add_value(XAPIAN_VALUE_ARCHIVE_ARCH, arches)
+            if apt_pkg.config.find("Apt::Architecture") not in arches:
+                return
         # Description (software-center extension)
         if parser.has_option_desktop("X-AppInstall-Description"):
             descr = parser.get_desktop("X-AppInstall-Description")
@@ -542,7 +666,7 @@ def index_app_info_from_parser(parser, db, cache):
             if k in globals():
                 w = globals()[k]
             else:
-                logging.debug("WEIGHT %s not found" % k)
+                LOG.debug("WEIGHT %s not found" % k)
                 w = 1
             term_generator.index_text_without_positions(s, w)
         # add data from the apt cache
@@ -569,19 +693,54 @@ def index_app_info_from_parser(parser, db, cache):
 
 def rebuild_database(pathname):
     cache = apt.Cache(memonly=True)
+    old_path = pathname+"_old"
+    rebuild_path = pathname+"_rb"
+    
+    if not os.path.exists(rebuild_path):
+        try:
+            os.makedirs(rebuild_path)
+        except:
+            LOG.warn("Problem creating rebuild path '%s'." % rebuild_path)
+            LOG.warn("Please check you have the relevant permissions.")
+            return False
+    
     # check permission
     if not os.access(pathname, os.W_OK):
         LOG.warn("Cannot write to '%s'." % pathname)
         LOG.warn("Please check you have the relevant permissions.")
         return False
+    
+    #check if old unrequired version of db still exists on filesystem
+    if os.path.exists(old_path):
+        LOG.warn("Existing xapian old db was not previously cleaned: '%s'." % old_path)
+        if os.access(old_path, os.W_OK):
+            #remove old unrequired db before beginning
+            shutil.rmtree(old_path)
+        else:
+            LOG.warn("Cannot write to '%s'." % old_path)
+            LOG.warn("Please check you have the relevant permissions.")
+            return False
+
+            
     # write it
-    db = xapian.WritableDatabase(pathname, xapian.DB_CREATE_OR_OVERWRITE)
+    db = xapian.WritableDatabase(rebuild_path, xapian.DB_CREATE_OR_OVERWRITE)
     update(db, cache)
+    # write the database version into the filep
+    db.set_metadata("db-schema-version", DB_SCHEMA_VERSION)
     # update the mo file stamp for the langpack checks
     mofile = gettext.find("app-install-data")
     if mofile:
         mo_time = os.path.getctime(mofile)
         db.set_metadata("app-install-mo-time", str(mo_time))
     db.flush()
-    return True
-
+    
+    # use shutil.move() instead of os.rename() as this will automatically
+    # figure out if it can use os.rename or needs to do the move "manually"
+    try:
+        shutil.move(pathname, old_path)
+        shutil.move(rebuild_path, pathname)
+        shutil.rmtree(old_path)
+        return True
+    except:
+        LOG.warn("Cannot copy refreshed database to correct location: '%s'." % pathname)
+        return False
